#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os.path
import wx

from core.mainElements import *
from core.config import *

from groupsCtrl import *
from smilesGroupCtrl import smilesFromGroup
from addgroup import *
from addsmile import *
from optionsDlg import *
from aboutDialog import *

class MainWindow(wx.Frame):
	def __init__(self, *args, **kwds):
		# begin wxGlade: MainWindow.__init__
		kwds["style"] = wx.DEFAULT_FRAME_STYLE
		wx.Frame.__init__(self, *args, **kwds)
		
		self.splitWindow = wx.SplitterWindow(self, -1, style=wx.SP_3D|wx.SP_BORDER)
		self.rightPanel = wx.Panel(self.splitWindow, -1)
		self.leftPanel = wx.Panel(self.splitWindow, -1)
		
		self.icon = wx.Icon(os.path.join (os.path.dirname(sys.argv[0]), \
										u"smilesarchive.ico"), wx.BITMAP_TYPE_ICO)
		self.SetIcon(self.icon)
		
		self.taskBarIcon = wx.TaskBarIcon()
		self.taskBarIcon.SetIcon(self.icon)
		
		self.makeMenu()

		self.groups = groupsCtrl(self.leftPanel, -1, (0, 0), (200, 200))
		self.smiles = smilesFromGroup (self.rightPanel, -1, (0, 0), (200, 200))

		self.__set_properties()
		self.__do_layout()
		
		self.taskBarIcon.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.OnTrayLeftClick)
		# end wxGlade		
		
		self.groups.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)
		
		self.SetSize (config.instance().mainWindowSize)
		self.splitWindow.SetSashPosition(config.instance().splitPosition)
		
		# Обработка событий
		self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGED, self.OnMoveSplit)
		self.Bind(wx.EVT_CLOSE, self.OnClose)
		self.Bind (wx.EVT_ICONIZE, self.OnMinimize)		

		self.__archive = archive()
		
		self.__archive.addGroupsObserver(self.groups)
		self.__archive.addSmilesObserver(self.smiles)
		self.__archive.addSmilesObserver(self.groups)
		self.__archive.load()
		
	def makeMenu (self):
		EXIT_MENU = 1
		ADDGROUP_MENU = 2
		ADDSMILE_MENU = 3
		ABOUT_MENU = 4
		REMOVE_GROUP_MENU = 5
		OPTIONS_MENU = 6
		
		# Menu Bar
		self.mainFrame_menubar = wx.MenuBar()
		self.SetMenuBar(self.mainFrame_menubar)
		
		#"File"
		file_menu = wx.Menu()
		file_menu.Append(EXIT_MENU, config.instance().localization[u"Exit\tCtrl+E"], "", wx.ITEM_NORMAL)
		self.mainFrame_menubar.Append(file_menu, config.instance().localization ["File"])
		
		#"Smiles"
		smiles_menu = wx.Menu()
		smiles_menu.Append(ADDSMILE_MENU, \
						config.instance().localization [u"Add smile...\tIns"], "", wx.ITEM_NORMAL)
		smiles_menu.AppendSeparator()
		smiles_menu.Append(ADDGROUP_MENU, \
						config.instance().localization [u"Add subgroup...\tCtrl+G"], "", wx.ITEM_NORMAL)
		smiles_menu.Append(REMOVE_GROUP_MENU, \
						config.instance().localization [u"Remove group...\tCtrl+R"], "", wx.ITEM_NORMAL)
		self.mainFrame_menubar.Append(smiles_menu, \
									config.instance().localization [u"Smiles"])
		
		#"Tools"
		tools_menu = wx.Menu()
		tools_menu.Append(OPTIONS_MENU, \
						config.instance().localization [u"Options...\tF8"], "", wx.ITEM_NORMAL)
		self.mainFrame_menubar.Append (tools_menu, \
									config.instance().localization [u"Tools"] )
		
		#"Help"
		help_menu = wx.Menu()
		help_menu.Append(ABOUT_MENU, \
						config.instance().localization [u"About...\tCtrl+F1"], "", wx.ITEM_NORMAL)
		self.mainFrame_menubar.Append(help_menu, \
									config.instance().localization [u"Help"])

		# Menu Bar end

		self.Bind(wx.EVT_MENU, self.OnAbout, id=ABOUT_MENU)
		self.Bind(wx.EVT_MENU, self.OnOptions, id=OPTIONS_MENU)
		self.Bind(wx.EVT_MENU, self.OnExit, id=EXIT_MENU)
		self.Bind(wx.EVT_MENU, self.OnAddGroup, id=ADDGROUP_MENU)
		self.Bind(wx.EVT_MENU, self.OnRemoveGroup, id=REMOVE_GROUP_MENU)
		self.Bind(wx.EVT_MENU, self.OnAddSmile, id=ADDSMILE_MENU)

	def __set_properties(self):
		# begin wxGlade: MainWindow.__set_properties
		self.SetTitle("SmilesArchive")
		self.groups.SetMinSize((186, 477))
		self.groups.SetFocus()
		self.smiles.SetMinSize((289, 477))
		self.splitWindow.SetMinSize((496, 481))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: MainWindow.__do_layout
		bottomSizer = wx.BoxSizer(wx.HORIZONTAL)
		smilesSizer = wx.BoxSizer(wx.HORIZONTAL)
		groupsSizer = wx.BoxSizer(wx.HORIZONTAL)
		groupsSizer.Add(self.groups, 1, wx.EXPAND, 0)
		self.leftPanel.SetSizer(groupsSizer)
		smilesSizer.Add(self.smiles, 1, wx.EXPAND, 0)
		self.rightPanel.SetSizer(smilesSizer)
		self.splitWindow.SplitVertically(self.leftPanel, self.rightPanel)
		bottomSizer.Add(self.splitWindow, 1, wx.EXPAND, 0)
		self.SetSizer(bottomSizer)
		bottomSizer.Fit(self)
		self.Layout()
		# end wxGlade

	def OnRemoveGroup (self, event):
		selgroup = self.__archive.getSelectedGroup()
		if selgroup == None:
			wx.MessageBox(config.instance().localization[u"Please, select group"], \
						config.instance().localization[u"Error"], wx.OK)
			return
		
		if selgroup.parent == None:
			wx.MessageBox(config.instance().localization[u"Can't to remove root group"],\
						config.instance().localization[u"Error"], wx.OK)
			return
		
		if wx.MessageBox(config.instance().localization[u"Really to remove group?"], \
						config.instance().localization[u"Remove group"], wx.YES_NO) == wx.YES:
			selgroup.parent.removeSubgroup (selgroup)

	def OnOptions (self, event):
		dlg = optionsDlg(None)
		dlg.ShowModal()
		dlg.Destroy()
	
	def OnExit(self, event): # wxGlade: MainWindow.<event_handler>
		self.Close()
			
	def OnAbout(self, event): # wxGlade: MainWindow.<event_handler>
		dlg = aboutDialog (None, -1, "")
		dlg.ShowModal()
		dlg.Destroy()

	def OnAddGroup(self, event): # wxGlade: MainWindow.<event_handler>
		selgroup = self.__archive.getSelectedGroup()
		if selgroup == None:
			wx.MessageBox(config.instance().localization[u"Please, select parent group"], config.instance().localization["Error"], wx.OK)
			return
		
		dlg = AddGroupDialog(None, -1, "")
		if dlg.ShowModal() == wx.ID_OK:
			name = dlg.nameText.GetValue()
			group = smilesGroup (name, self.__archive.getNextId(), [], [])
			selgroup.addSubGroup (group)
		dlg.Destroy()
			
			
	def OnAddSmile (self, event):
		selgroup = self.__archive.getSelectedGroup()
		if selgroup == None:
			wx.MessageBox(config.instance().localization[u"Please, select parent group"], config.instance().localization[u"Error"], wx.OK)
			return
		
		dlg = AddSmileDialog(None, -1, "")
		if dlg.ShowModal() == wx.ID_OK:
			url = dlg.urlText.GetValue()
			smile = smileInfo.load(url)
			
			if smile is not None:
				selgroup.addSmile (smile)
		dlg.Destroy()


	def OnSelChanged (self, event):
		selItem = self.groups.GetSelection()
		if selItem != None:
			data = self.groups.GetItemPyData(selItem)
			self.__archive.setSelectedGroup (data)
		
	def OnMoveSplit (self, event):
		config.instance().splitPosition = self.splitWindow.GetSashPosition()
		
	def OnClose (self, eventType):
		config.instance().mainWindowSize = self.GetSize()
		config.instance().splitPosition = self.splitWindow.GetSashPosition()
				
		config.instance().save()
		
		newphrasesfname = os.path.join(config.instance().langDir, config.instance().langName + "new")
		
		try:		
			config.instance().localization.saveNewPhrases (newphrasesfname)			
		finally:
			self.Destroy()
			if self.taskBarIcon.IsIconInstalled():
				self.taskBarIcon.RemoveIcon()
			
	def OnMinimize (self, eventType):
		if self.IsIconized():
			self.Hide()
				
			
	def OnTrayLeftClick (self, eventType):
		self.Show()
		self.Iconize(False)
		self.splitWindow.SetSashPosition (config.instance().splitPosition)

# end of class MainWindow

if __name__ == "__main__":
	app = wx.PySimpleApp(0)
	wx.InitAllImageHandlers()
	frame_1 = MainWindow(None, -1, "")
	app.SetTopWindow(frame_1)
	frame_1.Show()
	app.MainLoop()