# -*- coding: utf-8 -*-
from xml.dom.minidom import parse, Document
from string import *
from xml.parsers.expat import *
import os.path
import wx
import urllib2

from core.config import config

class smilesGroup:
	"""
	Информация о группе смайликов
	"""
	def __init__(self, name, id, smiles, subgroups):
		self.__name = name						# Имя группы
		self.__subgroups = subgroups			# Подгруппы
		self.__smiles = smiles					# Смайлики
		self.id = id
		self.__groupObservers = []				# Кто следит за изменением параметров группы
		self.__smilesObservers = []				# Кто следит за изменением смайликов
		self.parent = None
		
		# Установим родителей для подгрупп
		self.setSubParent (self.__subgroups, self)
		
		for smile in smiles:  
			smile.addObserver (self)
		
	def eventSmilesChange(self, smile):
	    self.onSmilesChange()
		
	def setSubParent (self, subgroups, parent):
		for group in subgroups:
			group.parent = parent
			group.setSubParent (group.getSubGroups(), group)

	def makeXmlElement (self, doc, tagname, text):
		node = doc.createElement (tagname)
		textVal = doc.createTextNode (text.encode ('utf-8').strip())
		node.appendChild (textVal)
		return node

	def toXml (self, doc):
		groupNode = doc.createElement (u'group')
		groupNode.appendChild (self.makeXmlElement(doc, u'name', self.__name) )
		groupNode.appendChild (self.makeXmlElement(doc, u'id', str (self.id) ) )
		
		smilesNode = doc.createElement (u'smiles')
		for smile in self.__smiles:
			smilesNode.appendChild (smile.toXml (doc))
		groupNode.appendChild (smilesNode)

		subgroups = doc.createElement (u'groups')
		for group in self.__subgroups:
			subgroups.appendChild (group.toXml (doc))			
		groupNode.appendChild (subgroups)
		
		return groupNode

	@staticmethod
	def fromXml (node):
		name = ""
		id = -1
		smiles = []
		groups = []
		
		for child in node.childNodes:
			# Имя группы
			if child.nodeName == "name":
				name = child.firstChild.nodeValue.strip()
			
			# id группы
			elif child.nodeName == "id":
				id = atoi (child.firstChild.nodeValue.strip())
			
			# Смайлики группы
			elif child.nodeName == "smiles":
				for smileNode in child.childNodes:
					if smileNode.nodeName == "smile":
						try:
							smile = smileInfo.fromXml(smileNode)
							smiles.append(smile)
						except ImportError:
							pass
			
			# Подгруппы
			elif child.nodeName == "groups":
				for groupNode in child.childNodes:
					if groupNode.nodeName == "group":
						group = smilesGroup.fromXml(groupNode)
						groups.append(group)
				
		return smilesGroup (name, id, smiles, groups)

	def getMaxId (self):
		"""
		Получить максимальное значение id во всех подгруппах. Нужно для поиска следующего id
		"""
		return self.recId(self, self.id)
		
	def recId (self, group, currmaxid):
		"""
		Для рекурсивного перебора id
		"""
		maxid = currmaxid
		for gr in group.getSubGroups():
			if gr.id > maxid:
				maxid = gr.id
			maxid = self.recId(gr, maxid)
		return maxid

	def addSmile (self, smile):
		self.__smiles.append(smile)
		smile.addObserver (self)
		self.onSmilesChange()
		
	def removeSmile (self, smile):
		os.remove (os.path.join (config.instance().cacheDir, smile.cacheName) )
		self.__smiles.remove (smile)
		self.onSmilesChange()
		
	def getName(self): 
		return self.__name
	
	def setName (self, value):
		self.__name = value
		self.onGroupChange()
		
	name = property (getName, setName)	

	def getSmiles(self): 
		return self.__smiles
	
	smiles = property (getSmiles)
	
	def getSubGroups (self):
		return self.__subgroups
	
	def addSubGroup (self, group):
		self.__subgroups.append(group)
		group.parent = self
		
		# Подпишем всех, кто следит за изменением в группе, на изменения в новой группе
		# На вложенные подгруппы не подписываем!
		for observer in self.__groupObservers:
			group.addGroupObserver(observer)
		
		for observer in self.__smilesObservers:
			group.addSmilesObserver(observer)
		
		self.onGroupChange()
		
	def clear (self):
		'''Очистить группу (удалить все смайлики и подгруппы'''
		for smile in self.__smiles:
			self.removeSmile (smile)
			
		for subgroup in self.__subgroups:
			subgroup.clear()
		
		
	def removeSubgroup (self, group):
		group.clear()
		self.__subgroups.remove (group)
		self.onGroupChange()
	
	# События об изменениях в группах
	def addGroupObserver (self, observer):
		"""
		Добавить наблюдателя за изменением параметров группы
		"""
		self.__groupObservers.append(observer)
		
		# Добавить наблюдателя к подгруппам
		for subgroup in self.__subgroups:
			subgroup.addGroupObserver (observer)
		
	def onGroupChange (self):
		for obs in self.__groupObservers:
			obs.eventGroupChange(self)

	# События об изменениях в смайлах	
	def addSmilesObserver (self, observer):
		"""
		Добавить наблюдателя за изменением параметров смайликов
		"""
		self.__smilesObservers.append(observer)
		
		# Добавить наблюдателя к подгруппам
		for subgroup in self.__subgroups:
			subgroup.addSmilesObserver (observer)
			
	def onSmilesChange (self):
		for obs in self.__smilesObservers:
			obs.eventSmilesChange(self)	
			


class smileInfo:
	"""
	Информация об одном смайлике
	"""
	def __init__ (self, url, size, cacheName):
		self.__url = url					# Адрес картинки
		self.size = size					# Размер, должен быть tuple
		self.__cacheName = cacheName		# Номер в кэше
		self.__observers = []				# Кто наблюдает за изменениями
		
	@staticmethod
	def load (url):
		if url.startswith ('http://') or url.startswith ('ftp://'):
			try:
				proxyurl = config.instance().proxyUrl
				if not proxyurl.startswith ('http://'):
				    proxyurl = u"http://" + proxyurl
					
				proxy = urllib2.ProxyHandler({'http': '%s:%d' % (proxyurl, \
					config.instance().proxyPort) })
				
				if config.instance().useProxy:
					opener = urllib2.build_opener(proxy)
				else:
					opener = urllib2.build_opener()
					
				opener.addheaders = [('User-agent', 'SmilesArchive')]
				fp = opener.open(url)
			except IOError:
				mess = wx.MessageDialog(None, config.instance().localization[u"Can't download smile"], \
					config.instance().localization[u"Error"], wx.OK)
				mess.ShowModal()
				return None
			except urllib2.URLError:
				mess = wx.MessageDialog(None, config.instance().localization[u"Invalid url"],\
					config.instance().localization[u"Error"], wx.OK)
				mess.ShowModal()
				fp.close()
				return None
		else:
			try:
				fp = file (url, 'rb')
			except IOError:
				mess = wx.MessageDialog(None, config.instance().localization[u"Can't open smile"],\
					config.instance().localization[u"Error"], wx.OK)
				mess.ShowModal()
				return None
		
		res = fp.read()
		fp.close()
		
		if not os.path.exists(config.instance().cacheDir):
			os.mkdir(config.instance().cacheDir)
		
		fname, fullfname = smileInfo.getSmileFileName (url)
		try:
			outfile = file (fullfname, "wb")
		except IOError:
			mess = wx.MessageDialog(None, config.instance().localization[u"Can't to save smile"],\
				config.instance().localization[u"Error"], wx.OK)
			mess.ShowModal()
			return None
		
		outfile.write(res)
		outfile.close()
		
		image = wx.Image (fullfname)
		res = smileInfo (url, image.GetSize(), fname)
		return res
	
	def setSize (self, size):
		self.size = size
		self.onChange()
	
	@staticmethod
	def getSmileFileName (url):
		
		fname = os.path.basename(url)
		
		dotindex = fname.rfind (".")
		if dotindex == -1:
			raise NameError()
		
		name = fname[: dotindex]
		ex = fname[dotindex:]
		
		fullname = os.path.join (config.instance().cacheDir, fname)
		
		if (os.path.exists(fullname)):
			index = 0
			while os.path.exists(fullname):
				fname = name + str (index) + ex
				fullname = os.path.join (config.instance().cacheDir, fname)
				index += 1
			
		return (fname, fullname)
		
	def addObserver (self, observer):
		self.__observers.append(observer)
		
	def onChange (self):
		for obs in self.__observers:
			obs.eventSmilesChange(self)


	def getUrl(self): 
		return self.__url
	
	url = property (getUrl)	
	
	
	def getCacheName (self):
		return self.__cacheName	
	
	cacheName = property (getCacheName)
		
	def makeXmlElement (self, doc, tagname, text):
		node = doc.createElement (tagname)
		textVal = doc.createTextNode (text.encode ('utf-8'))
		node.appendChild (textVal)
		return node
		
	def toXml (self, doc):
		smileElement = doc.createElement (u'smile')
		
		# Ссылка
		smileElement.appendChild (self.makeXmlElement(doc, u'url', self.__url))
		
		# Имя в кеше
		smileElement.appendChild (self.makeXmlElement(doc, u'cachename', self.__cacheName))
		
		# Ширина
		smileElement.appendChild (self.makeXmlElement(doc, u'width', str (self.size[0]) ) )
		
		# Высота
		smileElement.appendChild (self.makeXmlElement(doc, u'height', str (self.size[1]) ) )
		
		return smileElement
		
	@staticmethod
	def fromXml (node):
		url = ""
		cachename = ""
		width = 0
		height = 0
		
		for child in node.childNodes:
			if child.nodeName == "url":
				url = child.firstChild.nodeValue.strip()
				
			elif child.nodeName == "cachename":
				cachename = os.path.basename (child.firstChild.nodeValue.strip())
				
			elif child.nodeName == "width":
				try:
					width = atoi (child.firstChild.nodeValue.strip())
				except ValueError:
					pass
					
			elif child.nodeName == "height":
				try:
					height = atoi (child.firstChild.nodeValue.strip())
				except:
					pass
				
		if url == "" or cachename == "" or width == 0 or height == 0:
			raise ImportError
		
		return smileInfo (url, (width, height), cachename)

class archive:
	"""
	Архив всех групп
	"""
	def __init__(self):
		self.__root = smilesGroup(u"Smiles", 0, [], [])					# Список групп
		
		# Подпишемся на изменения
		self.__root.addGroupObserver(self)
		self.__root.addSmilesObserver(self)
				
		self.__selectedGroup = self.__root			# Текущая выбранная группа
		self.__groupsObservers = []					# Кто наблюдает за изменениями в архиве
		self.__smilesObservers = []					# Кто наблюдает за изменениями смайликов
	
	def toXml (self, doc):
		root = doc.createElement (u'archive')
		
		if self.__root != None:
			root.appendChild (self.__root.toXml(doc))
			
		return root

	def save (self):
		#self.backup()
		self.savetoFile (config.instance().archiveFileName)

	def savetoFile (self, fname):
		doc = Document ()
		doc.appendChild (self.toXml (doc))
		
		fp = file (fname, "wu")
		doc.writexml(fp, '\t', '\t', '\n', 'utf-8')
		fp.close()
		
	def fromXml (self, fname):
		try:
			doc = parse(fname)
			root = doc.documentElement
			for node in root.childNodes:
				if node.nodeName == u"group":
					self.__root = smilesGroup.fromXml(node)
					
			doc.unlink()
		except IOError:
			pass
		except ExpatError:
			self.__root = smilesGroup(u"Smiles", 0, [], [])
		
		self.__root.addGroupObserver(self)
		self.__root.addSmilesObserver(self)
		self.__selectedGroup = self.getRootGroup()
		self.onGroupsChange(self.__root)
		self.onSmilesChange(self.__root)
		
	def load (self):
		self.fromXml (config.instance().archiveFileName)

	def getSelectedGroup (self):
		return self.__selectedGroup
	
	def setSelectedGroup (self, value):
		self.__selectedGroup = value
		self.onSmilesChange(value)
		
	def eventGroupChange (self, sender):
		self.onGroupsChange(sender)
		
	def eventSmilesChange (self, sender):
		self.onSmilesChange(sender)
		
	def getRootGroup (self):
		return self.__root
	
	def getNextId (self):
		return self.__root.getMaxId() + 1
	
	def addGroupsObserver (self, observer):
		"""
		Добавить наблюдателя. Наблюдатель должен иметь метод eventGroupsChange  (self, sender, group)
		"""
		self.__groupsObservers.append (observer)
		
	def onGroupsChange (self, sender):
		"""
		Изменение в данных
		"""
		self.save()
		for obs in self.__groupsObservers:
			obs.eventGroupsChange(self, sender)
	
	
	def addSmilesObserver (self, observer):
		"""
		Добавить наблюдателя. Наблюдатель должен иметь метод eventSmilesChange (self, sender, group)
		"""
		self.__smilesObservers.append (observer)
		
	def onSmilesChange (self, sender):
		"""
		Изменение в смайликах
		"""
		self.save()
		for obs in self.__smilesObservers:
			obs.eventSmilesChange(self, sender)
	