# -*- coding: utf-8 -*-

import wx
import os.path

from core.config import *

class optionsDlg(wx.Dialog):
	"""Диалог с настройками"""
	
	def setparams (self):
		# Заполним параметры прокси
		self.useProxy.SetValue (config.instance().useProxy)
		
		self.proxyUrl.Enable (self.useProxy.GetValue() )
		self.proxyPort.Enable (self.useProxy.GetValue() )
		
		if config.instance().useProxy:
			self.proxyUrl.SetValue (config.instance().proxyUrl)	
			self.proxyPort.SetValue ( str (config.instance().proxyPort) )
		
		# Заполним список языков
		self.langbox.Append(u"English")
		self.langbox.Select(0)
		
		# Заполним языки
		dirnames = os.listdir(config.instance().langDir)
		index = 1
		for name in dirnames:
			if name.endswith (u".lng"):
				self.langbox.Append (name)
				if name == config.instance().langName:
					self.langbox.Select(index)

				index += 1	
					
	def __init__(self, *args, **kwds):
		kwds["style"] = wx.DEFAULT_DIALOG_STYLE
		wx.Dialog.__init__(self, *args, **kwds)
		
		self.USE_PROXY = 1
		
		# Выбор языка
		self.label_lang = wx.StaticText(self, -1, config.instance().localization[u"Language"])
		self.langbox = wx.ComboBox(self, -1, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY)
		
		# Контролы для прокси
		# Флажок "Использовать прокси"
		self.useProxy = wx.CheckBox (self, id = self.USE_PROXY, label = config.instance().localization[u"Use proxy"])
		self.useProxy.SetValue (config.instance().useProxy)
		
		# Адрес прокси
		self.proxyUrl_label = wx.StaticText(self, -1, config.instance().localization[u"URL"])
		self.proxyUrl = wx.TextCtrl (self, -1)
		self.proxyUrl.SetValue (config.instance().proxyUrl)
		
		# Порт для прокси
		self.proxyPort_label = wx.StaticText(self, -1, config.instance().localization[u"Port"])
		self.proxyPort = wx.TextCtrl (self, -1)
		self.proxyPort.SetValue (str (config.instance().proxyPort) )
		
		# Кнопки OK, Cancel
		self.ok = wx.Button(self, wx.ID_OK, "")
		#self.cancel = wx.Button(self, wx.ID_CANCEL, config.instance().localization[u"Cancel"])
		self.cancel = wx.Button(self, wx.ID_CANCEL, "")
		self.__set_properties()
		self.__do_layout()
		
		self.setparams()
			
		self.Bind(wx.EVT_BUTTON, self.onOk, id = wx.ID_OK)
		self.Bind(wx.EVT_CLOSE, self.onClose)
		
		self.Bind(wx.EVT_CHECKBOX, self.OnProxyClick, id = self.USE_PROXY)
				
	def onOk (self, event):
		# Выберем язык
		if self.langbox.GetSelection() == 0:
			config.instance().langName = u""
		else:
			config.instance().langName = self.langbox.GetStringSelection()
			
		# Сохраним настройки прокси
		config.instance().useProxy = self.useProxy.GetValue()
		
		if config.instance().useProxy:
			
			# Проверим на правильность ввода адреса прокси
			config.instance().proxyUrl = self.proxyUrl.GetValue()
			if len (self.proxyUrl.GetValue() ) == 0:
				wx.MessageBox(config.instance().localization[u"Empty proxy URL"], \
					config.instance().localization[u"Error"], wx.OK)
				return
			
			# Проверим на правильность ввода порта
			try:
				config.instance().proxyPort = int (self.proxyPort.GetValue())
			except  ValueError:
				wx.MessageBox(config.instance().localization[u"Invalid proxy port"], \
					config.instance().localization[u"Error"], wx.OK)
				return
			
		
		event.Skip()
		
	def OnProxyClick (self, event):
		self.proxyUrl.Enable (self.useProxy.GetValue() )
		self.proxyPort.Enable (self.useProxy.GetValue() )
		
	def __set_properties(self):
		# begin wxGlade: optionsDlg.__set_properties
		self.SetTitle(config.instance().localization[u"Options"])
		self.SetSize((400, 180))
		#self.langbox.SetMinSize((325, 21))
		self.ok.SetDefault()
		# end wxGlade

	def __do_layout(self):
		# Внутри него рсположены элементы для выбора языка
		#sizer_1 = wx.BoxSizer(wx.VERTICAL)
		
		# Внутри него расположены кнопки OK, Cancel
		sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
		
		grid_sizer_1 = wx.FlexGridSizer(5, 2, 0, 0)
		grid_sizer_1.Add(self.label_lang, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
		grid_sizer_1.Add(self.langbox, 0, wx.ALL|wx.EXPAND|wx.ALIGN_RIGHT, 4)
		
		# Использовать прокси?
		grid_sizer_1.Add(self.useProxy, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
		grid_sizer_1.AddStretchSpacer()
		
		# Адрес прокси
		grid_sizer_1.Add(self.proxyUrl_label, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
		grid_sizer_1.Add(self.proxyUrl, 0, wx.ALL|wx.EXPAND|wx.ALIGN_RIGHT, 4)
		
		# Порт прокси
		grid_sizer_1.Add(self.proxyPort_label, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
		grid_sizer_1.Add(self.proxyPort, 0, wx.ALL|wx.EXPAND|wx.ALIGN_RIGHT, 4)
		
		grid_sizer_1.AddGrowableCol(1)
		
		#sizer_1.Add(grid_sizer_1, 1, wx.TOP|wx.EXPAND, 0)
		sizer_2.Add(self.ok, 0, wx.ALL|wx.ALIGN_BOTTOM, 4)
		sizer_2.Add(self.cancel, 0, wx.ALL|wx.ALIGN_BOTTOM, 4)
		
		grid_sizer_1.AddStretchSpacer()
		grid_sizer_1.Add(sizer_2, 1, wx.ALL | wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT, 0)
		grid_sizer_1.AddGrowableRow(4)
		#sizer_1.Add(sizer_2, 1, wx.RIGHT|wx.ALIGN_RIGHT, 0)
		
		
		self.SetSizer(grid_sizer_1)
		self.Layout()

	def onClose (self, eventType):
		if __name__ == "__main__":
			self.Destroy()
		else:
			eventType.Skip()
		
# end of class optionsDlg

if __name__ == "__main__":
	app = wx.PySimpleApp(0)
	wx.InitAllImageHandlers()
	frame_1 = optionsDlg(None, -1, "")
	app.SetTopWindow(frame_1)
	frame_1.Show()
	app.MainLoop()

