﻿# -*- coding: utf-8 -*-
import os
import string

import wx.animate

from core.config import config
from smileInfoDlg import smileInfoDlg

class smileImage (wx.Panel):
	""" Класс для изображения смайлика"""	
	def __init__ (self, parent, smile, pos):		
		wx.Panel.__init__ (self, parent, -1, pos, (smile.size[0] + 6, smile.size[1] + 6), style = wx.SIMPLE_BORDER)
		
		self.smile = smile

		fname = self.getFileName(self.smile)
		
		# Проверим, гиф ли это
		if (self.isGif(fname)):
			self.gifSmile = wx.animate.GIFAnimationCtrl (self, -1, fname, \
				(2, 2))
			
			# Если в заголовке картинки неправильный размер
			realsize = self.gifSmile.GetEffectiveMinSize()
			if smile.size[0] != realsize[0] or smile.size[1] != realsize[1]:
				#print smile.size
				smile.setSize (realsize)
			
			self.gifSmile.Play()
		else:
			self.bmp = wx.Bitmap (fname)
			
		self.Bind(wx.EVT_CLOSE, self.OnClose)
		self.Bind(wx.EVT_PAINT, self.OnPaint)
		  
	def getFileName (self, smile):
		fname = os.path.join (config.instance().cacheDir, smile.cacheName)
		if not os.path.exists(fname):
			fname = os.path.join (config.instance().imagesDir, "error.png")
		return fname
		
	def isGif (self, fname):
		index = string.lower(fname).rfind (".gif")
		return index == len (fname) - 4
	
	def OnClose (self, eventType):
		if self.isGif (self.getFileName (self.smile) ):
			self.gifSmile.Stop()
			self.gifSmile.Close()
			self.gifSmile.Hide()
		self.Hide()
		self.Destroy()

	def OnPaint (self, event):
		dc = wx.PaintDC (self)
		if not self.isGif (self.getFileName (self.smile) ):
			dc.DrawBitmap(self.bmp, 2, 2, False)
	

class smileButton (smileImage):
	""" Смайлик с контекстным меню"""
	def __init__ (self, parent, smile, smileGroup, pos):
		smileImage.__init__ (self, parent, smile, pos)
		
		fname = self.getFileName(self.smile)
		if (self.isGif(fname)):
			self.gifSmile.Bind(wx.EVT_RIGHT_UP, self.OnRmouseClick)
		
		self.smileGroup = smileGroup	
		self.setupMenu()
			
	def setupMenu(self):
		""" Установим контекстное меню """
		self.COPY_HTML = 11
		self.COPY_BBCODE = 12
		self.COPY_URL = 13
		self.REMOVE_SMILE = 14
		self.INFO = 15			
		
		self.menu = wx.Menu ()
		
		self.menu.Append(self.COPY_HTML, config.instance().localization[u"Copy HTML code to clipboard"])
		self.menu.Append(self.COPY_BBCODE, config.instance().localization[u"Copy BB-code to clipboard"])
		self.menu.Append(self.COPY_URL, config.instance().localization[u"Copy URL image to clipboard"])
	
		self.menu.AppendSeparator()
		self.menu.Append(self.INFO, config.instance().localization[u"Information..."])
		
		self.menu.AppendSeparator()
		self.menu.Append(self.REMOVE_SMILE, config.instance().localization[u"Remove smile..."])		
		
		self.Bind(wx.EVT_RIGHT_UP, self.OnRmouseClick)		
		self.Bind(wx.EVT_MENU, self.OnHtmlCopy, id = self.COPY_HTML)
		self.Bind(wx.EVT_MENU, self.OnBbcodeCopy, id = self.COPY_BBCODE)
		self.Bind(wx.EVT_MENU, self.OnUrlCopy, id = self.COPY_URL)		
		self.Bind(wx.EVT_MENU, self.OnRemoveSmile, id = self.REMOVE_SMILE)		
		self.Bind(wx.EVT_MENU, self.OnInfoSmile, id = self.INFO)		
		
	def OnInfoSmile(self, event):
		""" Вызов диалога с информацией о смайлике """
		dlg = smileInfoDlg (self.smile, None, id = -1, \
			title = config.instance().localization[u"Information"])
		dlg.ShowModal()
		dlg.Destroy()
	
	def OnRemoveSmile (self, eventType):
		if wx.MessageBox(config.instance().localization[u"Really to remove a smile?"], config.instance().localization[u"Remove smile?"], style = wx.YES_NO) == wx.YES:
			self.smileGroup.removeSmile (self.smile)

	def OnRmouseClick (self, event):
		self.PopupMenu (self.menu)
		
	def OnHtmlCopy (self, event):
		text = "<img src='%s' width='%d' height='%d' border='0'/>" % (self.smile.getUrl(), 
																	self.smile.size[0],
																	self.smile.size[1])
		cb = wx.TheClipboard
		if not cb.Open():
			print config.instance().localization[u"Can't open clipboard"]
			return
		
		data = wx.TextDataObject (text)		
		cb.SetData(data)
		cb.Flush()
		cb.Close()
		
	def OnBbcodeCopy (self, event):
		text = "[img]%s[/img]" % (self.smile.getUrl())
		cb = wx.TheClipboard
		if not cb.Open():
			print config.instance().localization[u"Can't open clipboard"]
			return
		
		data = wx.TextDataObject (text)		
		cb.SetData(data)
		cb.Flush()
		cb.Close()
		
	def OnUrlCopy (self, event):
		cb = wx.TheClipboard
		if not cb.Open():
			print config.instance().localization[u"Can't open clipboard"]
			return
		
		data = wx.TextDataObject (self.smile.getUrl())		
		cb.SetData(data)
		cb.Flush()
		cb.Close()
	

class smilesFromGroup (wx.ScrolledWindow):
	def __init__ (self, parent, id, pos, size):
		wx.ScrolledWindow.__init__ (self, parent, id, pos, size)
		self.SetScrollRate (10, 10)
		self.smiles = list()
		self.currentGroup = None
		
		self.Bind(wx.EVT_SIZE, self.OnSize)
		
	def OnSize (self, size):
		self.updateSmiles()
		
	def updateSmiles(self):
		for smileCtrl in self.smiles:
			smileCtrl.Close()
			#smileCtrl.Destroy()
			
			# TODO: Как удалить окно?
		
		del self.smiles[:]
		#self.smiles = list()
		
		w = self.Size[0]
		h = self.Size[1]
		
		maxheight = 0
		margin = 8
		
		currx = 0
		curry = 0
		
		if self.currentGroup != None:
			for smile in self.currentGroup.smiles:
				smilew = smile.size[0]
				#
				if smilew + currx + margin >= w:
					currx = 0
					curry += maxheight + margin
					maxheight = 0
				
				button = smileButton (self, smile, self.currentGroup, (currx, curry))
				button.Show(True)
				#button.Layout()
				#button.SetSize (button.GetAdjustedBestSize())
				
				self.smiles.append(button)
				currx += smile.size[0] + margin
				
				# Найдем максимальную высоту смайлика в строке
				if smile.size[1] > maxheight:
					maxheight = smile.size[1]
					
			self.Scroll(0, 0)
			self.SetVirtualSize( (self.ClientSize[0], curry + maxheight) )
		
	def eventSmilesChange (self, sender, group):
		self.currentGroup = group
		self.updateSmiles()