﻿# -*- coding: utf-8 -*-
import unittest
import os
import sys

from core.mainElements import *
from gui.addsmile import *

class smileInfoTest (unittest.TestCase):
	"""
	Тест класса smileInfo
	"""
	def testCreate(self):
		info = smileInfo('http://localhost/test.gif', (16, 16), "test.gif")
		self.assertEqual (info.url, 'http://localhost/test.gif')
		self.assertEqual (info.size, (16, 16) )
		self.assertEqual (info.cacheName, "test.gif")
		
	def testLoad (self):
		smile = smileInfo.load (u'http://jenyay.net/uploads/Images/HomePage/beer.gif')
		self.assertEqual (smile.url, u'http://jenyay.net/uploads/Images/HomePage/beer.gif')
		self.assertEqual (smile.size[0], 57)
		self.assertEqual (smile.size[1], 16)
		
class smilesGroupTest (unittest.TestCase):
	"""
	Тест класса smilesGroup
	"""
	def testCreate (self):
		group = smilesGroup('groupname', 0, [], [])
		self.assertEqual (group.name, 'groupname')
		
	def setUp (self):
		self.groupEventCount = 0;
		self.smilesEventCount = 0;
		
	def eventGroupChange (self, sender):
		self.groupEventCount += 1

	def eventSmilesChange (self, sender):
		self.smilesEventCount += 1
		
	def testEventGroup (self):
		root = smilesGroup ('root', 0, [], [])
		root.addGroupObserver(self)
		
		group2 = smilesGroup ('group2', 0, [], [])
		root.addSubGroup(group2)
		self.assertEqual (self.groupEventCount, 1)
		
		group3 = smilesGroup ('group3', 0, [], [])
		group2.addSubGroup(group3)
		self.assertEqual (self.groupEventCount, 2)
		
		group4 = smilesGroup ('group4', 0, [], [])
		group3.addSubGroup(group4)
		self.assertEqual (self.groupEventCount, 3)
		
		group5 = smilesGroup ('group5', 0, [], [])
		root.addSubGroup(group2)
		self.assertEqual (self.groupEventCount, 4)
		
	def testMaxId (self):
		root = smilesGroup('groupname', 0, [], [])
		group1 = smilesGroup('group1', 1, [], [])
		root.addSubGroup(group1)
		self.assertEqual (root.getMaxId(), 1)
		
		group2 = smilesGroup('group2', 2, [], [])
		root.addSubGroup(group2)
		self.assertEqual (root.getMaxId(), 2)
		
		group3 = smilesGroup('group3', 3, [], [])
		group2.addSubGroup(group3)
		self.assertEqual (root.getMaxId(), 3)
		
		group4 = smilesGroup('group4', 4, [], [])
		group3.addSubGroup(group4)
		self.assertEqual (root.getMaxId(), 4)
		
		group5 = smilesGroup('group5', 5, [], [])
		group3.addSubGroup(group5)
		self.assertEqual (root.getMaxId(), 5)
			
	def testEventSmiles (self):
		root = smilesGroup ('root', 0, [], [])
		root.addSmilesObserver(self)
		
		group2 = smilesGroup ('group2', 0, [], [])
		root.addSubGroup(group2)
		self.assertEqual (self.smilesEventCount, 0)

	def testAddSmile (self):
		self.smilesEventCount = 0
		
		root = smilesGroup ('root', 0, [], [])
		root.addSmilesObserver(self)
		smile1 = smileInfo ('http://localhost/test.gif', (16, 20), "test.gif")
				
		self.assertEqual (len (root.smiles), 0)
		
		root.addSmile (smile1)
		self.assertEqual (self.smilesEventCount, 1)
		self.assertEqual (len (root.smiles), 1)
		
		group2 = smilesGroup ('group2', 0, [], [])
		root.addSubGroup (group2)
		smile2 = smileInfo ('http://localhost/test2.gif', (16, 20), "test.gif")
		group2.addSmile(smile2)
		self.assertEqual (self.smilesEventCount, 2)
		self.assertEqual (len (group2.smiles), 1)
		
		smile3 = smileInfo ('http://localhost/test3.gif', (20, 25), "test.gif")
		group2.addSmile(smile3)
		self.assertEqual (self.smilesEventCount, 3)
		self.assertEqual (len (group2.smiles), 2)
		
	def testRemoveSmile (self):
		self.smilesEventCount = 0
		
		root = smilesGroup ('root', 0, [], [])
		
		path = os.path.join ('tests', 'test.gif')
		fname = os.path.join(os.path.dirname(sys.argv[0]), path);
		print fname
		
		smile1 = smileInfo.load (fname)
		smile2 = smileInfo.load (fname)
		smile3 = smileInfo.load (fname)
		
		#print smile1.cacheName
		#print smile2.cacheName
		
		root.addSmile(smile1)
		root.addSmile(smile2)
		root.addSmile(smile3)
		
		self.smilesEventCount = 0
		root.addSmilesObserver(self)
		root.removeSmile (smile2)
		
		self.assertEqual (self.smilesEventCount, 1)
		self.assertEqual (len (root.getSmiles()), 2)
		
		self.assertEqual (root.getSmiles()[0], smile1)
		self.assertEqual (root.getSmiles()[1], smile3)
		
		root.removeSmile (smile1)
		root.removeSmile (smile3)
		
class archiveTest (unittest.TestCase):
	"""
	Тест класса archive
	"""
	def setUp (self):
		self.groupEventCount = 0
		self.smilesEventCount = 0
		
	def testCreate (self):
		a = archive()
		self.assertEqual (len (a.getRootGroup().getSubGroups()), 0)
		

#	def testEventsAfterXml (self):
#		arch = archive()
#		arch.addGroupsObserver(self)
#		arch.addSmilesObserver(self)		
#		fname = os.path.join (os.path.dirname(sys.argv[0]), "archive.smiles")
#		arch.fromXml(fname)
#		
#		self.groupEventCount = 0
#		self.smilesEventCount = 0
#		
#		group2_1 = arch.getRootGroup().getSubGroups()[0].getSubGroups()[1]
#		newsmile = smileInfo ("http://localhost", (20, 25), "newsmile.gif")
#		group2_1.addSmile (newsmile)
#		self.assertEqual (self.smilesEventCount, 1)
#		
#		newgroup = smilesGroup ("newgroup", 111, [], [])
#		group2_1.addSubGroup (newgroup)
#		self.assertEqual (self.groupEventCount, 1)
		
	def testNotingXml (self):
		arch = archive()
		arch.fromXml("none.xml")
		self.assertEqual (arch.getRootGroup().name, unicode ("Smiles", "utf-8"))
		self.assertEqual (len (arch.getRootGroup().getSubGroups()), 0)
		self.assertEqual (len (arch.getRootGroup().getSmiles()), 0)
		
	def testInvalidXml (self):
		arch = archive()
		fname = os.path.join (os.path.dirname(sys.argv[0]), "invalid1.xml")
		arch.fromXml(fname)
		self.assertEqual (arch.getRootGroup().name, unicode ("Smiles", "utf-8"))
		self.assertEqual (len (arch.getRootGroup().getSubGroups()), 0)
		self.assertEqual (len (arch.getRootGroup().getSmiles()), 0)		
			
	# def testInvalidSmileXml (self):
		# arch = archive()
		# fname = os.path.join (os.path.dirname(sys.argv[0]), "tests/invalidsmile.xml")
		# arch.fromXml(fname)
		# self.assertEqual (arch.getRootGroup().name, u'archive')
		# self.assertEqual (len (arch.getRootGroup().getSubGroups()), 0)
		# self.assertEqual (len (arch.getRootGroup().getSmiles()), 1)
		# self.assertEqual (arch.getRootGroup().getSmiles()[0].url, "fullsmile")
		
	def testSelectGroup(self):
		arch = archive()
		fname = os.path.join (os.path.dirname(sys.argv[0]), "archive.smiles")
		arch.fromXml(fname)
		arch.addSmilesObserver(self)
		self.smilesEventCount = 0
		
		arch.setSelectedGroup (arch.getRootGroup())
		self.assertEqual (self.smilesEventCount, 1)
		self.assertEqual (arch.getSelectedGroup(), arch.getRootGroup())
		
		
	def eventGroupsChange  (self, sender, group):
		self.groupEventCount += 1
		
	def eventSmilesChange (self, sender, group):
		self.smilesEventCount += 1
		
	def testXml (self):
		# Создадим архив
		arch = archive()
		
		group1 = smilesGroup (u'Группа1', 0, [], [])		
		smile1_1 = smileInfo (u'path1', (10, 20), u'smile1_1')
		smile2_1 = smileInfo (u'path2', (10, 20), u'smile2_1')
		group1.addSmile (smile1_1)
		group1.addSmile (smile2_1)
		
		group2 = smilesGroup (u'Группа2', 0, [], [])		
		smile1_2 = smileInfo (u'path3', (10, 20), u'smile1_2')
		group2.addSmile (smile1_2)
		
		group3 = smilesGroup (u'Группа3', 0, [], [])		
		smile1_3 = smileInfo (u'path5', (10, 20), u'smile1_3')
		smile2_3 = smileInfo (u'path6', (10, 20), u'smile2_3')
		group3.addSmile (smile1_3)
		group3.addSmile (smile2_3)
		
		group1.addSubGroup(group2)
		group1.addSubGroup(group3)
		
		arch.getRootGroup().addSubGroup (group1)
		arch.save ()
		
		arch2 = archive()
		arch2.load()
		
		root2 = arch2.getRootGroup()
		root = arch.getRootGroup()		 
		self.assertEqual (root.name, root2.name)
		
		sub1_1 = root.getSubGroups()
		sub1_2 = root2.getSubGroups() 
		self.assertEqual (sub1_1[0].name, sub1_2[0].name)
		self.assertEqual (len (sub1_1), len (sub1_2))
		
		smile1_1_2 = sub1_2[0].getSmiles()[0]
		self.assertEqual (smile1_1_2.url, smile1_1.url)
		
		
class addSmileDialogTest(unittest.TestCase):
	def testExtensions (self):
		self.assertTrue (AddSmileDialog.isGoodExtension("test.gif"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.Gif"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.png"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.PnG"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.jpg"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.Jpg"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.jpeg"))
		self.assertTrue (AddSmileDialog.isGoodExtension("test.JpEG"))
		
		self.assertFalse (AddSmileDialog.isGoodExtension("test.gyf"))

if __name__ == '__main__':
	config.instance ('tests')
	unittest.main()
